#!/usr/bin/env bash
# R4iSDHC/R4i-SDHC Flashcart Kernel Install Tool

clear
echo "R4 Downloader: *nix Edition"
echo "The user-friendly installer for your R4iSDHC/R4i-SDHC cart."
echo "Copyright © 2023 Yoshi128k. All rights reserved"
sleep 2
read -rep $'\nPress RETURN to continue . . .'

while true
do
    clear
    printf "Select an option:\n\n"
    echo "1. Download Stock Kernel (if you dare)"
    echo "2. Download RGFMCU (YSMenu)"
    echo "3. Exit"
    read -rep $'\nType a number and press RETURN: ' opt

    case $opt in

        1)
            clear
            printf "Which domain is displayed on the label of your flashcart:\n"
            echo "1. r4isdhc.com"
            echo "2. r4i-sdhc.com"
            read -rep $'\nType a number and press RETURN: ' urlNum

            case $urlNum in
                1)
                    if [[ ! -f "./r4i-40b-eng.zip" ]]; then
                        clear
                        echo "Ready to download R4iSDHC 20XX kernel, version 4.0b"
                        echo "Oh, and by the way, the files will be downloaded to the directory where the downloader is located."
                        read -rep $'\nPress RETURN to continue . . .'
                    
                        echo "Starting download . . ."
                        curl "https://gbatemp.net/download/kernel-v4-0b-for-r4-sdhc-gold-pro-rts-lite-dual-core-snoopy-upgrade-r4isdhc-com.37072/download" -o "./r4i-40b-eng.zip"
                        echo "Done!"
                        sleep 1
                    else
                        echo ".zip file already exists, skipping download."
                        sleep 1
                    fi
                    
                    clear
                    echo "[Installation]"
                    echo "If you have not already done so, mount the microSD card now."
                    echo "Make sure it is formatted as FAT32. If not, do so before continuing."
                    while true
                    do
                        read -rep $'\nType the mount point of the card and press RETURN: ' mountPoint
                        read -p "${mountPoint}: Is this correct (Y/N): " choice
                        echo ${choice,,} 
                        case ${choice,,} in
                            y)
                                break
                            ;;

                            n)
                                continue
                            ;;
                        esac
                    done
                    
                    echo "Extracting . . ."
                    7za x -aoa r4i-40b-eng.zip -r
                    echo "Done!"
                    sleep 1
                    
                    echo "Copying to microSD card . . . "
                    cp -rv ./R4i-V4.0b\ English/*  $mountPoint
                    echo "Done!"
                    sleep 1
                ;;
                2)
                    if [[ ! -f "./r4i-185b-eng.zip" ]]; then
                        clear
                        echo "Ready to download R4i-SDHC kernel, version 1.85b"
                        echo "Oh, and by the way, the files will be downloaded to the directory where the downloader is located."
                        read -rep $'\nPress RETURN to continue . . .'
                    
                        echo "Starting download . . ."
                        curl "https://gbatemp.net/download/firmware-v1-85b-for-r4-sdhc-upgrade-r4i-sdhc-upgrade-r4i-3ds-www-r4i-sdhc-com-english-clone-fix.37690/download" -o "./r4i-185b-eng.zip"
                        echo "Done!"
                        sleep 1
                    else
                        echo ".zip file already exists, skipping download."
                        sleep 1
                    fi

                    clear
                    echo "[Installation]"
                    echo "If you have not already done so, mount the microSD card now."
                    echo "Make sure it is formatted as FAT32. If not, do so before continuing."
                    while true
                    do
                        read -rep $'\nType the mount point of the card and press RETURN: ' mountPoint
                        read -p "${mountPoint}: Is this correct (Y/N): " choice
                        echo ${choice,,} 
                        case ${choice,,} in
                            y)
                                break
                            ;;

                            n)
                                continue
                            ;;
                        esac
                    done
                    
                    echo "Extracting . . ."
                    7za x -aoa r4i-3ds-v1.85b-eng-fix.zip -o* -r
                    echo "Done!"
                    sleep 1
                    
                    echo "Copying to microSD card . . . "
                    cp -rv ./r4i-3ds-v1.85b-eng-fix/* $mountPoint
                    echo "Done!"
                    sleep 1
                ;;
            esac
            clear
            echo "Install complete!"
            echo "Enjoy your newly-setup flashcart!"
            echo "NOTE: The stock kernels have a timebomb (2024-09-03)."
            echo "After this date, you will need to set your system's clock to some time before it, otherwise you will get a \"firmware expired\" error on startup."
            read -rep $'\nAny more cards to set up? (Y/N): ' choice
            case ${choice,,} in
                y)
                    continue
                ;;
                n)
                    break
                ;;
            esac
        ;;
        2)
            if [[ ! -f "RGFMCU.7z" ]]; then
                clear
                echo "Ready to download the RetroGameFan Multi-Cart Update, version 7.06"
                echo "Oh, and by the way, the files will be downloaded to the directory where the downloader is located."
                read -rep $'\nPress RETURN to continue . . .'
                    
                echo "Starting download . . ."
                curl "https://gbatemp.net/download/retrogamefan-multi-cart-update.35737/download" -o "./RGFMCU.7z"
                echo "Done!"
                sleep 1
            else
                echo ".7z file already exists, skipping download."
            fi
            clear
            echo "[Installation]"
            echo "If you have not already done so, mount the microSD card now."
            echo "Make sure it is formatted as FAT32. If not, do so before continuing."
            while true
            do
                read -rep $'\nType the mount point of the card and press RETURN: ' mountPoint
                read -p "${mountPoint}: Is this correct (Y/N): " choice
                echo ${choice,,} 
                case ${choice,,} in
                    y)
                        break
                    ;;

                    n)
                        continue
                    ;;
                esac
            done
            echo "Extracting . . ."
            7za x -aoa -r RGFMCU.7z "RetroGameFan Multi Cart Update v7.06/R4i-SDHC YSMenu/"
            sleep 1

            clear
            echo "Chose one of the installation options below:"
            echo "1. Primary Kernel (replaces stock)"
            echo "2. Dual Boot (stock kernel first)"
            echo "3. Dual Boot (YSMenu first)"
            echo "NOTE: the /"Dual Boot/" options require that the stock kernel be installed."
            read -rep $'\nType a number and press RETURN: ' opt
            case $opt in
                1)
                    echo "Copying files . . ."
                    cp -rv "./RetroGameFan Multi Cart Update v7.06/R4i-SDHC YSMenu/TTMenu" $mountPoint
                    cp -rv "./RetroGameFan Multi Cart Update v7.06/R4i-SDHC YSMenu/Games" $mountPoint
                    cp -rv "./RetroGameFan Multi Cart Update v7.06/R4i-SDHC YSMenu/R4.dat" $mountPoint
                    cp -rv "./RetroGameFan Multi Cart Update v7.06/R4i-SDHC YSMenu/TTMenu.dat" $mountPoint
                    echo "Done!"
                    sleep 1
                ;;
                2)
                    echo "Copying files . . ."
                    cp -rv "./RetroGameFan Multi Cart Update v7.06/R4i-SDHC YSMenu/TTMenu" $mountPoint
                    cp -rv "./RetroGameFan Multi Cart Update v7.06/R4i-SDHC YSMenu/Games" $mountPoint
                    cp -rv "./RetroGameFan Multi Cart Update v7.06/R4i-SDHC YSMenu/YSMenu.nds" $mountPoint
                    cp -rv "./RetroGameFan Multi Cart Update v7.06/R4i-SDHC YSMenu/TTMenu.dat" $mountPoint
                    echo "Done!"
                    sleep 1
                ;;
                3)
                    echo "Copying files . . ."
                    mv $mountPoint/R4.dat $mountPoint/R4Stock.nds
                    cp -rv "./RetroGameFan Multi Cart Update v7.06/R4i-SDHC YSMenu/TTMenu" $mountPoint
                    cp -rv "./RetroGameFan Multi Cart Update v7.06/R4i-SDHC YSMenu/Games" $mountPoint
                    cp -rv "./RetroGameFan Multi Cart Update v7.06/R4i-SDHC YSMenu/R4.dat" $mountPoint
                    cp -rv "./RetroGameFan Multi Cart Update v7.06/R4i-SDHC YSMenu/TTMenu.dat" $mountPoint
                    echo "Done!"
                    sleep 1
                ;;
            esac
            clear
            echo "Install complete!"
            echo "Enjoy your newly-setup flashcart!"
            echo "NOTE: If you chose the \"Dual Boot (YSMenu first)\" option, the stock kernel has been renamed \"R4Stock.nds.\" Run that to launch the stock kernel."
            read -rep $'\nAny more cards to set up? (Y/N): ' choice
            case ${choice,,} in
                y)
                    continue
                ;;
                n)
                    break
                ;;
            esac
            
        ;;
        3)
            break
        ;;
        *)
            echo "Error: Invalid choice"
            exit 1
        ;;

    esac
done
echo "Goodbye!"
exit 0